@@ -1,6 +1,6 @@
 #[cfg(feature = "compile-with-external-structures")]
 pub(crate) mod external {
-    use crate::blobs::Blob;
+    use crate::blobs::{Blob, HasBlob};
     use std::ops::Deref;
 
     /// C-compatible list
@@ -12,9 +12,10 @@ pub(crate) mod external {
     extern "C" {
         fn lib_ruby_parser__external__string_ptr__new(ptr: *const u8, len: u64) -> Blob<StringPtr>;
         fn lib_ruby_parser__external__string_ptr__drop(blob: *mut Blob<StringPtr>);
-        fn lib_ruby_parser__external__string_ptr__get_raw(
-            blob_ptr: *mut Blob<StringPtr>,
-        ) -> *mut u8;
+        fn lib_ruby_parser__external__string_ptr__as_raw(
+            blob_ptr: *const Blob<StringPtr>,
+        ) -> *const u8;
+        fn lib_ruby_parser__external__string_ptr__into_raw(blob_ptr: Blob<StringPtr>) -> *mut u8;
         fn lib_ruby_parser__external__string_ptr__get_len(blob: *const Blob<StringPtr>) -> u64;
     }
 
@@ -44,6 +45,11 @@ pub(crate) mod external {
     }
 
     impl StringPtr {
+        pub(crate) fn from_raw(ptr: *const u8, len: u64) -> Self {
+            let blob = unsafe { lib_ruby_parser__external__string_ptr__new(ptr, len) };
+            Self { blob }
+        }
+
         /// Performs uppercase on self. Returns Err if stored byte array is invalid in UTF-8
         pub fn to_uppercase(&self) -> Self {
             Self::from(self.as_str().to_uppercase())
@@ -60,21 +66,17 @@ pub(crate) mod external {
             unsafe { lib_ruby_parser__external__string_ptr__get_len(&self.blob) as usize }
         }
 
+        /// Equivalent of String::is_empty
+        pub fn is_empty(&self) -> bool {
+            self.len() == 0
+        }
+
         pub(crate) fn as_ptr(&self) -> *const u8 {
-            let blob_ptr: *const Blob<StringPtr> = &self.blob;
-            unsafe {
-                lib_ruby_parser__external__string_ptr__get_raw(blob_ptr as *mut Blob<StringPtr>)
-            }
+            unsafe { lib_ruby_parser__external__string_ptr__as_raw(&self.blob) }
         }
 
-        pub(crate) fn as_mut_ptr(&mut self) -> *mut u8 {
-            unsafe { lib_ruby_parser__external__string_ptr__get_raw(&mut self.blob) }
-        }
-
-        pub(crate) fn into_ptr(mut self) -> *mut u8 {
-            let ptr = self.as_mut_ptr();
-            std::mem::forget(self);
-            ptr
+        pub(crate) fn into_ptr(self) -> *mut u8 {
+            unsafe { lib_ruby_parser__external__string_ptr__into_raw(self.into_blob()) }
         }
     }
 
@@ -115,8 +117,7 @@ pub(crate) mod external {
                 bytes.as_ptr()
             };
 
-            let blob = unsafe { lib_ruby_parser__external__string_ptr__new(ptr, len) };
-            Self { blob }
+            Self::from_raw(ptr, len)
         }
     }
 
@@ -148,14 +149,6 @@ pub(crate) mod external {
         }
     }
 
-    impl From<&mut StringPtr> for &mut [u8] {
-        fn from(value: &mut StringPtr) -> Self {
-            let ptr = value.as_mut_ptr();
-            let len = value.len();
-            unsafe { std::slice::from_raw_parts_mut(ptr, len) }
-        }
-    }
-
     impl PartialEq<&str> for StringPtr {
         fn eq(&self, other: &&str) -> bool {
             self.as_ref() == other.as_bytes()
