@@ -1,14 +1,15 @@
 crate::use_native_or_external!(Ptr);
 crate::use_native_or_external!(StringPtr);
 crate::use_native_or_external!(List);
+crate::use_native_or_external!(Maybe);
 
 use crate::debug_level;
 use crate::lexer::*;
 use crate::maybe_byte::*;
 use crate::source::buffer::*;
 use crate::source::Comment;
+use crate::source::Decoder;
 use crate::source::MagicComment;
-use crate::source::MaybeDecoder;
 use crate::str_term::{str_types::*, HeredocEnd, StrTerm, StringLiteral};
 use crate::Context;
 use crate::Loc;
@@ -33,7 +34,7 @@ pub struct Lexer {
     pub(crate) lval_start: Option<usize>,
     pub(crate) lval_end: Option<usize>,
 
-    pub(crate) strterm: Option<StrTerm>,
+    pub(crate) strterm: Option<Box<StrTerm>>,
     /// Current state of the lexer, used internally for testing
     pub lex_state: LexState,
     pub(crate) paren_nest: i32,
@@ -49,8 +50,7 @@ pub struct Lexer {
 
     pub(crate) tokenbuf: TokenBuf,
 
-    pub(crate) max_numparam: usize,
-
+    // pub(crate) max_numparam: usize,
     pub(crate) context: Context,
     pub(crate) in_kwarg: bool,
 
@@ -97,7 +97,7 @@ impl Lexer {
     pub(crate) const VTAB_CHAR: u8 = 0x0b;
 
     /// Constructs an instance of Lexer
-    pub fn new<Bytes, Name>(bytes: Bytes, name: Name, decoder: MaybeDecoder) -> Self
+    pub fn new<Bytes, Name>(bytes: Bytes, name: Name, decoder: Maybe<Decoder>) -> Self
     where
         Bytes: Into<List<u8>>,
         Name: Into<StringPtr>,
@@ -139,7 +139,7 @@ impl Lexer {
     }
 
     pub(crate) fn yylex(&mut self) -> Ptr<Token> {
-        let lex_state_before = self.lex_state.clone();
+        let lex_state_before = self.lex_state;
         self.lval = None;
 
         let token_type = self.parser_yylex();
@@ -154,12 +154,12 @@ impl Lexer {
                 // take raw value if nothing was manually captured
                 self.buffer
                     .substr_at(begin, end)
-                    .map(|s| Bytes::new(s.to_vec()))
+                    .map(|s| Bytes::new(List::from(s)))
             })
-            .unwrap_or_else(|| Bytes::new(b"".to_vec()));
+            .unwrap_or_else(|| Bytes::new(list![]));
 
         if token_type == Self::tNL {
-            token_value = Bytes::new(b"\n".to_vec());
+            token_value = Bytes::new(list![b'\n']);
             end = begin + 1;
         }
 
@@ -199,7 +199,7 @@ impl Lexer {
         let mut last_state: LexState;
         let token_seen = self.token_seen;
 
-        if let Some(strterm) = self.strterm.as_ref() {
+        if let Some(strterm) = self.strterm.as_ref().map(|i| i.as_ref()) {
             match strterm {
                 StrTerm::HeredocLiteral(_) => {
                     return self.here_document();
@@ -217,7 +217,7 @@ impl Lexer {
         self.token_seen = true;
 
         'retrying: loop {
-            last_state = self.lex_state.clone();
+            last_state = self.lex_state;
             self.token_flush();
 
             // handle EOF
@@ -227,7 +227,7 @@ impl Lexer {
                 return Self::END_OF_INPUT;
             }
 
-            match c.to_option() {
+            match c.as_option() {
                 None
                 | Some(Self::NULL_CHAR)
                 | Some(Self::CTRL_D_CHAR)
@@ -284,6 +284,7 @@ impl Lexer {
                         // while(1)
                         c = self.nextc();
 
+                        #[allow(clippy::never_loop)]
                         // emulate ugly C switch with fall-through logic
                         loop {
                             if c == b' '
@@ -297,13 +298,13 @@ impl Lexer {
                             }
 
                             if c == b'#' {
-                                self.buffer.pushback(&c);
+                                self.buffer.pushback(c);
                                 continue 'retrying;
                             }
 
                             if c == b'&' || c == b'.' {
                                 if self.buffer.peek(b'.') == (c == b'&') {
-                                    self.buffer.pushback(&c);
+                                    self.buffer.pushback(c);
                                     continue 'retrying;
                                 }
                             }
@@ -336,8 +337,8 @@ impl Lexer {
                             self.lex_state.set(EXPR_BEG);
                             return Self::tOP_ASGN;
                         }
-                        self.buffer.pushback(&c);
-                        if self.lex_state.is_spacearg(&c, space_seen) {
+                        self.buffer.pushback(c);
+                        if self.lex_state.is_spacearg(c, space_seen) {
                             self.warn(
                                 DiagnosticMessage::new_d_star_interpreted_as_arg_prefix(),
                                 self.current_loc(),
@@ -350,9 +351,9 @@ impl Lexer {
                                 Self::tPOW,
                                 "**",
                                 "argument prefix",
-                                &c,
+                                c,
                                 space_seen,
-                                &last_state,
+                                last_state,
                             );
                         }
                     } else {
@@ -361,8 +362,8 @@ impl Lexer {
                             self.lex_state.set(EXPR_BEG);
                             return Self::tOP_ASGN;
                         }
-                        self.buffer.pushback(&c);
-                        if self.lex_state.is_spacearg(&c, space_seen) {
+                        self.buffer.pushback(c);
+                        if self.lex_state.is_spacearg(c, space_seen) {
                             self.warn(
                                 DiagnosticMessage::new_star_interpreted_as_arg_prefix(),
                                 self.current_loc(),
@@ -375,9 +376,9 @@ impl Lexer {
                                 Self::tSTAR2,
                                 "*",
                                 "argument prefix",
-                                &c,
+                                c,
                                 space_seen,
-                                &last_state,
+                                last_state,
                             );
                         }
                     }
@@ -406,7 +407,7 @@ impl Lexer {
                     if c == b'~' {
                         return Self::tNMATCH;
                     }
-                    self.buffer.pushback(&c);
+                    self.buffer.pushback(c);
                     return Self::tBANG;
                 }
 
@@ -429,7 +430,7 @@ impl Lexer {
                                 if c == b'=' && self.buffer.is_word_match("end") {
                                     break;
                                 }
-                                self.buffer.pushback(&c);
+                                self.buffer.pushback(c);
                             }
                             self.buffer.goto_eol();
                             self.comments.push(Comment::new(
@@ -451,7 +452,7 @@ impl Lexer {
                         if c == b'=' {
                             return Self::tEQQ;
                         }
-                        self.buffer.pushback(&c);
+                        self.buffer.pushback(c);
                         return Self::tEQ;
                     }
                     if c == b'~' {
@@ -459,7 +460,7 @@ impl Lexer {
                     } else if c == b'>' {
                         return Self::tASSOC;
                     }
-                    self.buffer.pushback(&c);
+                    self.buffer.pushback(c);
                     return Self::tEQL;
                 }
 
@@ -489,7 +490,7 @@ impl Lexer {
                         if c == b'>' {
                             return Self::tCMP;
                         }
-                        self.buffer.pushback(&c);
+                        self.buffer.pushback(c);
                         return Self::tLEQ;
                     }
                     if c == b'<' {
@@ -499,17 +500,17 @@ impl Lexer {
                             self.lex_state.set(EXPR_BEG);
                             return Self::tOP_ASGN;
                         }
-                        self.buffer.pushback(&c);
+                        self.buffer.pushback(c);
                         return self.warn_balanced(
                             Self::tLSHFT,
                             "<<",
                             "here document",
-                            &c,
+                            c,
                             space_seen,
-                            &last_state,
+                            last_state,
                         );
                     }
-                    self.buffer.pushback(&c);
+                    self.buffer.pushback(c);
                     return Self::tLT;
                 }
 
@@ -532,10 +533,10 @@ impl Lexer {
                             self.lex_state.set(EXPR_BEG);
                             return Self::tOP_ASGN;
                         }
-                        self.buffer.pushback(&c);
+                        self.buffer.pushback(c);
                         return Self::tRSHFT;
                     }
-                    self.buffer.pushback(&c);
+                    self.buffer.pushback(c);
                     return Self::tGT;
                 }
 
@@ -594,7 +595,7 @@ impl Lexer {
                             self.lex_state.set(EXPR_BEG);
                             return Self::tOP_ASGN;
                         }
-                        self.buffer.pushback(&c);
+                        self.buffer.pushback(c);
                         return Self::tANDOP;
                     } else if c == b'=' {
                         self.set_yylval_id("&=");
@@ -605,8 +606,8 @@ impl Lexer {
                         self.lex_state.set(EXPR_DOT);
                         return Self::tANDDOT;
                     }
-                    self.buffer.pushback(&c);
-                    if self.lex_state.is_spacearg(&c, space_seen) {
+                    self.buffer.pushback(c);
+                    if self.lex_state.is_spacearg(c, space_seen) {
                         if c != b':'
                             || {
                                 c = self.buffer.peekc_n(1);
@@ -631,9 +632,9 @@ impl Lexer {
                             Self::tAMPER2,
                             "&",
                             "argument prefix",
-                            &c,
+                            c,
                             space_seen,
-                            &last_state,
+                            last_state,
                         );
                     }
                     self.lex_state.set(if self.lex_state.is_after_operator() {
@@ -654,9 +655,9 @@ impl Lexer {
                             self.lex_state.set(EXPR_BEG);
                             return Self::tOP_ASGN;
                         }
-                        self.buffer.pushback(&c);
+                        self.buffer.pushback(c);
                         if last_state.is_some(EXPR_BEG) {
-                            self.buffer.pushback(&Some(b'|'));
+                            self.buffer.pushback(b'|');
                             return Self::tPIPE;
                         }
                         return Self::tOROP;
@@ -671,7 +672,7 @@ impl Lexer {
                     } else {
                         EXPR_BEG | EXPR_LABEL
                     });
-                    self.buffer.pushback(&c);
+                    self.buffer.pushback(c);
                     return Self::tPIPE;
                 }
 
@@ -682,7 +683,7 @@ impl Lexer {
                         if c == b'@' {
                             return Self::tUPLUS;
                         }
-                        self.buffer.pushback(&c);
+                        self.buffer.pushback(c);
                         return Self::tPLUS;
                     }
                     if c == b'=' {
@@ -691,25 +692,25 @@ impl Lexer {
                         return Self::tOP_ASGN;
                     }
                     if self.lex_state.is_beg()
-                        || (self.lex_state.is_spacearg(&c, space_seen)
+                        || (self.lex_state.is_spacearg(c, space_seen)
                             && self.arg_ambiguous(b'+', self.current_loc().adjust_end(-1)))
                     {
                         self.lex_state.set(EXPR_BEG);
-                        self.buffer.pushback(&c);
+                        self.buffer.pushback(c);
                         if !c.is_eof() && c.is_digit() {
                             return self.parse_numeric(b'+');
                         }
                         return Self::tUPLUS;
                     }
                     self.lex_state.set(EXPR_BEG);
-                    self.buffer.pushback(&c);
+                    self.buffer.pushback(c);
                     return self.warn_balanced(
                         Self::tPLUS,
                         "+",
                         "unary operator",
-                        &c,
+                        c,
                         space_seen,
-                        &last_state,
+                        last_state,
                     );
                 }
 
@@ -720,7 +721,7 @@ impl Lexer {
                         if c == b'@' {
                             return Self::tUMINUS;
                         }
-                        self.buffer.pushback(&c);
+                        self.buffer.pushback(c);
                         return Self::tMINUS;
                     }
                     if c == b'=' {
@@ -733,25 +734,25 @@ impl Lexer {
                         return Self::tLAMBDA;
                     }
                     if self.lex_state.is_beg()
-                        || (self.lex_state.is_spacearg(&c, space_seen)
+                        || (self.lex_state.is_spacearg(c, space_seen)
                             && self.arg_ambiguous(b'-', self.current_loc().adjust_end(-1)))
                     {
                         self.lex_state.set(EXPR_BEG);
-                        self.buffer.pushback(&c);
+                        self.buffer.pushback(c);
                         if !c.is_eof() && c.is_digit() {
                             return Self::tUMINUS_NUM;
                         }
                         return Self::tUMINUS;
                     }
                     self.lex_state.set(EXPR_BEG);
-                    self.buffer.pushback(&c);
+                    self.buffer.pushback(c);
                     return self.warn_balanced(
                         Self::tMINUS,
                         "-",
                         "unary operator",
-                        &c,
+                        c,
                         space_seen,
-                        &last_state,
+                        last_state,
                     );
                 }
 
@@ -775,10 +776,10 @@ impl Lexer {
                             }
                             return if is_beg { Self::tBDOT3 } else { Self::tDOT3 };
                         }
-                        self.buffer.pushback(&c);
+                        self.buffer.pushback(c);
                         return if is_beg { Self::tBDOT2 } else { Self::tDOT2 };
                     }
-                    self.buffer.pushback(&c);
+                    self.buffer.pushback(c);
                     if !c.is_eof() && c.is_digit() {
                         let prev = if self.buffer.pcur - 1 > self.buffer.pbeg {
                             self.buffer.byte_at(self.buffer.pcur - 2)
@@ -800,7 +801,7 @@ impl Lexer {
                     return Self::tDOT;
                 }
 
-                Some(c) if c >= b'0' && c <= b'9' => {
+                Some(c) if (b'0'..=b'9').contains(&c) => {
                     return self.parse_numeric(c);
                 }
 
@@ -844,7 +845,7 @@ impl Lexer {
                             || self.lex_state.is_some(EXPR_CLASS)
                             || self
                                 .lex_state
-                                .is_spacearg(&MaybeByte::EndOfInput, space_seen)
+                                .is_spacearg(MaybeByte::EndOfInput, space_seen)
                         {
                             self.lex_state.set(EXPR_BEG);
                             return Self::tCOLON3;
@@ -854,26 +855,26 @@ impl Lexer {
                         return Self::tCOLON2;
                     }
                     if self.lex_state.is_end() || c.is_space() || c == Some(b'#') {
-                        self.buffer.pushback(&c);
+                        self.buffer.pushback(c);
                         let result = self.warn_balanced(
                             Self::tCOLON,
                             ":",
                             "symbol literal",
-                            &c,
+                            c,
                             space_seen,
-                            &last_state,
+                            last_state,
                         );
                         self.lex_state.set(EXPR_BEG);
                         return result;
                     }
-                    match c.to_option() {
+                    match c.as_option() {
                         Some(c) if c == b'\'' => {
                             self.strterm = self.new_strterm(str_ssym, c, None, None)
                         }
                         Some(c) if c == b'"' => {
                             self.strterm = self.new_strterm(str_dsym, c, None, None)
                         }
-                        _ => self.buffer.pushback(&c),
+                        _ => self.buffer.pushback(c),
                     }
                     self.lex_state.set(EXPR_FNAME);
                     return Self::tSYMBEG;
@@ -890,8 +891,8 @@ impl Lexer {
                         self.lex_state.set(EXPR_BEG);
                         return Self::tOP_ASGN;
                     }
-                    self.buffer.pushback(&c);
-                    if self.lex_state.is_spacearg(&c, space_seen) {
+                    self.buffer.pushback(c);
+                    if self.lex_state.is_spacearg(c, space_seen) {
                         self.arg_ambiguous(b'/', self.current_loc());
                         self.strterm = self.new_strterm(str_regexp, b'/', None, None);
                         return Self::tREGEXP_BEG;
@@ -905,9 +906,9 @@ impl Lexer {
                         Self::tDIVIDE,
                         "/",
                         "regexp literal",
-                        &c,
+                        c,
                         space_seen,
-                        &last_state,
+                        last_state,
                     );
                 }
 
@@ -923,7 +924,7 @@ impl Lexer {
                     } else {
                         EXPR_BEG
                     });
-                    self.buffer.pushback(&c);
+                    self.buffer.pushback(c);
                     return Self::tCARET;
                 }
 
@@ -942,7 +943,7 @@ impl Lexer {
                     if self.lex_state.is_after_operator() {
                         c = self.nextc();
                         if c != b'@' {
-                            self.buffer.pushback(&c);
+                            self.buffer.pushback(c);
                         }
                         self.lex_state.set(EXPR_ARG);
                     } else {
@@ -991,10 +992,10 @@ impl Lexer {
                             if c == b'=' {
                                 return Self::tASET;
                             }
-                            self.buffer.pushback(&c);
+                            self.buffer.pushback(c);
                             return Self::tAREF;
                         }
-                        self.buffer.pushback(&c);
+                        self.buffer.pushback(c);
                         self.lex_state.set(EXPR_ARG | EXPR_LABEL);
                         return Self::tLBRACK2;
                     } else if self.lex_state.is_beg()
@@ -1049,7 +1050,7 @@ impl Lexer {
                         return Self::tSP;
                     }
                     if c.is_space() {
-                        match c.to_option() {
+                        match c.as_option() {
                             Some(b'\t') => return Self::tSLASH_T,
                             Some(Self::LF_CHAR) => return Self::tSLASH_F,
                             Some(b'\r') => return Self::tSLASH_R,
@@ -1058,7 +1059,7 @@ impl Lexer {
                             None => {}
                         }
                     }
-                    self.buffer.pushback(&c);
+                    self.buffer.pushback(c);
                     return Self::tBACKSLASH;
                 }
 
@@ -1099,13 +1100,13 @@ impl Lexer {
             break;
         }
 
-        self.parse_ident(&c, cmd_state)
+        self.parse_ident(c, cmd_state)
     }
 
     fn normal_newline_leaf_label(&mut self) -> i32 {
         self.command_start = true;
         self.lex_state.set(EXPR_BEG);
-        return Self::tNL;
+        Self::tNL
     }
 
     pub(crate) fn warn(&mut self, message: DiagnosticMessage, loc: Loc) {
@@ -1121,9 +1122,9 @@ impl Lexer {
         token_type: i32,
         op: &'static str,
         syn: &'static str,
-        c: &MaybeByte,
+        c: MaybeByte,
         space_seen: bool,
-        last_state: &LexState,
+        last_state: LexState,
     ) -> i32 {
         if !last_state.is_some(EXPR_CLASS | EXPR_DOT | EXPR_FNAME | EXPR_ENDFN)
             && space_seen & !c.is_space()
@@ -1150,14 +1151,14 @@ impl Lexer {
         term: u8,
         paren: Option<u8>,
         heredoc_end: Option<HeredocEnd>,
-    ) -> Option<StrTerm> {
-        Some(StrTerm::new_literal(StringLiteral::new(
+    ) -> Option<Box<StrTerm>> {
+        Some(Box::new(StrTerm::new_literal(StringLiteral::new(
             0,
             func,
             paren,
             term,
             heredoc_end,
-        )))
+        ))))
     }
 
     pub(crate) fn loc(&self, begin_pos: usize, end_pos: usize) -> Loc {
@@ -1201,10 +1202,9 @@ impl Lexer {
         self.lpar_beg == self.paren_nest
     }
 
-    pub(crate) fn tokadd_ident(&mut self, c: &MaybeByte) -> bool {
-        let mut c = c.clone();
+    pub(crate) fn tokadd_ident(&mut self, mut c: MaybeByte) -> bool {
         loop {
-            if self.tokadd_mbchar(&c).is_err() {
+            if self.tokadd_mbchar(c).is_err() {
                 return true;
             }
             c = self.nextc();
@@ -1214,7 +1214,7 @@ impl Lexer {
             }
         }
 
-        self.buffer.pushback(&c);
+        self.buffer.pushback(c);
         false
     }
 
@@ -1228,7 +1228,7 @@ impl Lexer {
         self.buffer.set_ptok(ptok);
     }
 
-    pub(crate) fn tokadd_mbchar(&mut self, c: &MaybeByte) -> Result<(), ()> {
+    pub(crate) fn tokadd_mbchar(&mut self, c: MaybeByte) -> Result<(), ()> {
         let mut len = match self.multibyte_char_len(self.buffer.pcur - 1) {
             Some(len) => len,
             None => return Err(()),
@@ -1246,7 +1246,7 @@ impl Lexer {
     }
 
     fn _multibyte_char_len(&self, ptr: usize) -> Option<usize> {
-        let c1 = self.buffer.byte_at(ptr).to_option()?;
+        let c1 = self.buffer.byte_at(ptr).as_option()?;
 
         let len = if c1 & 0x80 == 0 {
             1
