@@ -3,12 +3,8 @@ use crate::source::buffer::*;
 use crate::DiagnosticMessage;
 use crate::{lex_states::*, LexState};
 
-pub(crate) trait ParseAtMark {
-    fn parse_atmark(&mut self, last_state: LexState) -> i32;
-}
-
-impl ParseAtMark for Lexer {
-    fn parse_atmark(&mut self, last_state: LexState) -> i32 {
+impl Lexer {
+    pub(crate) fn parse_atmark(&mut self, last_state: LexState) -> i32 {
         let ptr = self.buffer.pcur;
         let mut result: i32 = Self::tIVAR;
         let mut c = self.nextc();
@@ -27,7 +23,7 @@ impl ParseAtMark for Lexer {
             EXPR_END
         });
         if c.is_eof() || !self.is_identchar() {
-            self.buffer.pushback(&c);
+            self.buffer.pushback(c);
             if result == Self::tIVAR {
                 self.compile_error(DiagnosticMessage::new_ivar_without_id(), self.current_loc());
             } else {
@@ -53,7 +49,7 @@ impl ParseAtMark for Lexer {
             return result;
         }
 
-        if self.tokadd_ident(&c) {
+        if self.tokadd_ident(c) {
             return Self::END_OF_INPUT;
         }
         self.tokenize_ident();
