@@ -1,41 +1,16 @@
 use crate::lexer::TokAdd;
-use crate::lexer::Yylval;
 use crate::maybe_byte::*;
 use crate::source::buffer::*;
 use crate::Lexer;
 use crate::TokenBuf;
 use crate::{lex_states::*, DiagnosticMessage};
 
-pub(crate) trait ParseNumeric {
-    fn parse_numeric(&mut self, prefix: u8) -> i32;
-    fn parse_octal(
-        &mut self,
-        c: &mut MaybeByte,
-        nondigit: &mut Option<MaybeByte>,
-        start: usize,
-    ) -> Option<i32>;
-    fn invalid_octal(&mut self) -> i32;
-    fn trailing_uc(&mut self, nondigit: u8) -> i32;
-    fn decode_num(
-        &mut self,
-        c: MaybeByte,
-        nondigit: Option<MaybeByte>,
-        is_float: bool,
-        seen_e: bool,
-    ) -> i32;
-    fn parse_numeric_footer(&mut self, is_float: bool, seen_e: bool) -> i32;
-    fn set_number_literal(&mut self, value: &mut TokenBuf, token_type: i32, suffix: i8) -> i32;
-    fn no_digits(&mut self) -> i32;
-    fn number_literal_suffix(&mut self, mask: i8) -> i8;
-    fn set_integer_literal(&mut self, value: &mut TokenBuf, suffix: i8) -> i32;
-}
-
 const NUM_SUFFIX_R: i8 = 1 << 0;
 const NUM_SUFFIX_I: i8 = 1 << 1;
 const NUM_SUFFIX_ALL: i8 = 3;
 
-impl ParseNumeric for Lexer {
-    fn parse_numeric(&mut self, prefix: u8) -> i32 {
+impl Lexer {
+    pub(crate) fn parse_numeric(&mut self, prefix: u8) -> i32 {
         let mut c = MaybeByte::new(prefix);
 
         let mut is_float: bool = false;
@@ -47,7 +22,7 @@ impl ParseNumeric for Lexer {
         self.lex_state.set(EXPR_END);
         self.newtok();
         if c == b'-' || c == b'+' {
-            self.tokadd(&c);
+            self.tokadd(c);
             c = self.nextc();
         }
         if c == b'0' {
@@ -55,7 +30,7 @@ impl ParseNumeric for Lexer {
             c = self.nextc();
             if c == b'x' || c == b'X' {
                 // hexadecimal
-                self.tokadd(&c);
+                self.tokadd(c);
                 c = self.nextc();
                 if !c.is_eof() && c.is_hexdigit() {
                     loop {
@@ -63,8 +38,8 @@ impl ParseNumeric for Lexer {
                             if nondigit.is_some() {
                                 break;
                             }
-                            nondigit = Some(c.clone());
-                            self.tokadd(&c);
+                            nondigit = Some(c);
+                            self.tokadd(c);
                             c = self.nextc();
                             if c.is_eof() {
                                 break;
@@ -75,7 +50,7 @@ impl ParseNumeric for Lexer {
                             break;
                         }
                         nondigit = None;
-                        self.tokadd(&c);
+                        self.tokadd(c);
 
                         c = self.nextc();
                         if c.is_eof() {
@@ -83,7 +58,7 @@ impl ParseNumeric for Lexer {
                         }
                     }
                 }
-                self.buffer.pushback(&c);
+                self.buffer.pushback(c);
                 self.tokfix();
                 if self.toklen() == start + 1 {
                     return self.no_digits();
@@ -97,7 +72,7 @@ impl ParseNumeric for Lexer {
             }
             if c == b'b' || c == b'B' {
                 // binary
-                self.tokadd(&c);
+                self.tokadd(c);
                 c = self.nextc();
                 if c == b'0' || c == b'1' {
                     loop {
@@ -105,8 +80,8 @@ impl ParseNumeric for Lexer {
                             if nondigit.is_some() {
                                 break;
                             }
-                            nondigit = Some(c.clone());
-                            self.tokadd(&c);
+                            nondigit = Some(c);
+                            self.tokadd(c);
                             c = self.nextc();
                             if c.is_eof() {
                                 break;
@@ -117,7 +92,7 @@ impl ParseNumeric for Lexer {
                             break;
                         }
                         nondigit = None;
-                        self.tokadd(&c);
+                        self.tokadd(c);
 
                         c = self.nextc();
                         if c.is_eof() {
@@ -125,7 +100,7 @@ impl ParseNumeric for Lexer {
                         }
                     }
                 }
-                self.buffer.pushback(&c);
+                self.buffer.pushback(c);
                 self.tokfix();
                 if self.toklen() == start + 1 {
                     return self.no_digits();
@@ -139,7 +114,7 @@ impl ParseNumeric for Lexer {
             }
             if c == b'd' || c == b'D' {
                 // decimal
-                self.tokadd(&c);
+                self.tokadd(c);
                 c = self.nextc();
                 if !c.is_eof() && c.is_digit() {
                     loop {
@@ -147,8 +122,8 @@ impl ParseNumeric for Lexer {
                             if nondigit.is_some() {
                                 break;
                             }
-                            nondigit = Some(c.clone());
-                            self.tokadd(&c);
+                            nondigit = Some(c);
+                            self.tokadd(c);
                             c = self.nextc();
                             if c.is_eof() {
                                 break;
@@ -159,7 +134,7 @@ impl ParseNumeric for Lexer {
                             break;
                         }
                         nondigit = None;
-                        self.tokadd(&c);
+                        self.tokadd(c);
 
                         c = self.nextc();
                         if c.is_eof() {
@@ -167,7 +142,7 @@ impl ParseNumeric for Lexer {
                         }
                     }
                 }
-                self.buffer.pushback(&c);
+                self.buffer.pushback(c);
                 self.tokfix();
                 if self.toklen() == start + 1 {
                     return self.no_digits();
@@ -186,13 +161,15 @@ impl ParseNumeric for Lexer {
                 }
             }
             if c == b'o' || c == b'O' {
-                self.tokadd(&c);
+                self.tokadd(c);
                 // prefixed octal
                 c = self.nextc();
                 if c.is_eof() || c == b'_' || !c.is_digit() {
                     return self.no_digits();
                 }
             }
+            // `c` here is a MaybeByte that implements PartialOrd<u8>
+            #[allow(clippy::manual_range_contains)]
             if c >= b'0' && c <= b'7' {
                 // octal
                 if let Some(result) = self.parse_octal(&mut c, &mut nondigit, start) {
@@ -204,7 +181,7 @@ impl ParseNumeric for Lexer {
             } else if c == b'.' || c == b'e' || c == b'E' {
                 self.tokadd(b'0');
             } else {
-                self.buffer.pushback(&c);
+                self.buffer.pushback(c);
                 suffix = self.number_literal_suffix(NUM_SUFFIX_ALL);
 
                 let mut tok = self.tokenbuf.take();
@@ -214,11 +191,11 @@ impl ParseNumeric for Lexer {
         }
 
         loop {
-            match c.to_option() {
+            match c.as_option() {
                 Some(b'0') | Some(b'1') | Some(b'2') | Some(b'3') | Some(b'4') | Some(b'5')
                 | Some(b'6') | Some(b'7') | Some(b'8') | Some(b'9') => {
                     nondigit = None;
-                    self.tokadd(&c);
+                    self.tokadd(c);
                 }
 
                 Some(b'.') => {
@@ -230,38 +207,38 @@ impl ParseNumeric for Lexer {
                     } else {
                         let c0 = self.nextc();
                         if c.is_eof() || !c0.is_digit() {
-                            self.buffer.pushback(&c0);
+                            self.buffer.pushback(c0);
                             return self.decode_num(c, nondigit, is_float, seen_e);
                         }
                         c = c0;
                     }
                     seen_point = Some(self.toklen());
                     self.tokadd(b'.');
-                    self.tokadd(&c);
+                    self.tokadd(c);
                     is_float = true;
                     nondigit = None;
                 }
 
                 Some(b'e') | Some(b'E') => {
                     if let Some(nondigit_value) = &nondigit {
-                        self.buffer.pushback(&c);
-                        c = nondigit_value.clone();
+                        self.buffer.pushback(c);
+                        c = *nondigit_value;
                         return self.decode_num(c, nondigit, is_float, seen_e);
                     }
                     if seen_e {
                         return self.decode_num(c, nondigit, is_float, seen_e);
                     }
-                    nondigit = Some(c.clone());
+                    nondigit = Some(c);
                     c = self.nextc();
                     if c != b'-' && c != b'+' && !c.is_digit() {
-                        self.buffer.pushback(&c);
+                        self.buffer.pushback(c);
                         nondigit = None;
                         return self.decode_num(c, nondigit, is_float, seen_e);
                     }
-                    self.tokadd(&nondigit.clone().expect("nondigit must be set"));
+                    self.tokadd(nondigit.expect("nondigit must be set"));
                     seen_e = true;
                     is_float = true;
-                    self.tokadd(&c);
+                    self.tokadd(c);
                     nondigit = if c == b'-' || c == b'+' {
                         Some(c)
                     } else {
@@ -270,7 +247,7 @@ impl ParseNumeric for Lexer {
                 }
 
                 Some(b'_') => {
-                    self.tokadd(&c);
+                    self.tokadd(c);
                     if nondigit.is_some() {
                         return self.decode_num(c, nondigit, is_float, seen_e);
                     }
@@ -295,8 +272,8 @@ impl ParseNumeric for Lexer {
                 if nondigit.is_some() {
                     break;
                 }
-                *nondigit = Some(c.clone());
-                self.tokadd(&*c);
+                *nondigit = Some(*c);
+                self.tokadd(*c);
                 *c = self.nextc();
                 if c.is_eof() {
                     break;
@@ -311,7 +288,7 @@ impl ParseNumeric for Lexer {
                 return None;
             }
             *nondigit = None;
-            self.tokadd(&*c);
+            self.tokadd(*c);
 
             *c = self.nextc();
             if c.is_eof() {
@@ -356,7 +333,7 @@ impl ParseNumeric for Lexer {
         is_float: bool,
         seen_e: bool,
     ) -> i32 {
-        self.buffer.pushback(&c);
+        self.buffer.pushback(c);
         if let Some(MaybeByte::Some(byte)) = nondigit {
             self.trailing_uc(byte);
         }
@@ -435,7 +412,7 @@ impl ParseNumeric for Lexer {
                 // self.literal_flush(self.buffer.pcur);
                 return 0;
             }
-            self.buffer.pushback(&c);
+            self.buffer.pushback(c);
             break;
         }
 
