@@ -7,54 +7,21 @@ use crate::TokenBuf;
 use crate::{lex_states::*, DiagnosticMessage};
 use crate::{lexer::*, str_term::StringLiteral};
 
-pub(crate) trait ParseString {
-    fn take_strterm(&mut self) -> StringLiteral;
-    fn restore_strterm(&mut self, lit: StringLiteral);
-
-    fn parse_string(&mut self) -> i32;
-    fn string_term(&mut self, term: u8, func: usize) -> i32;
-    fn regx_options(&mut self) -> String;
-    fn peek_variable_name(&mut self) -> Option<i32>;
-    fn tokadd_string(
-        &mut self,
-        func: usize,
-        term: u8,
-        paren: Option<u8>,
-        nest: &mut usize,
-    ) -> Option<MaybeByte>;
-    fn flush_string_content(&mut self);
-    fn tokadd_utf8_unterminated(&mut self);
-    fn scan_hex(&mut self, start: usize, len: usize, numlen: &mut usize) -> usize;
-    fn scan_oct(&mut self, start: usize, len: usize, numlen: &mut usize) -> usize;
-    fn tokcopy(&mut self, n: usize);
-    fn tokaddmbc(&mut self, codepoint: usize);
-    fn tokadd_codepoint(&mut self, regexp_literal: usize, wide: bool) -> bool;
-    fn tokadd_utf8(&mut self, term: Option<u8>, _symbol_literal: usize, regexp_literal: usize);
-    fn simple_re_meta(&mut self, c: &MaybeByte) -> bool;
-    fn tokadd_escape_eof(&mut self) -> Result<(), ()>;
-    fn tokadd_escape(&mut self) -> Result<(), ()>;
-    fn read_escape_eof(&mut self) -> MaybeByte;
-    fn tok_hex(&mut self, numlen: &mut usize) -> MaybeByte;
-    fn read_escape(&mut self, flags: usize) -> MaybeByte;
-    fn is_ascii(&self) -> bool;
-    fn warn_space_char(&mut self, c: u8, prefix: &'static str);
-}
-
 const ESCAPE_CONTROL: usize = 1;
 const ESCAPE_META: usize = 2;
 
-impl ParseString for Lexer {
+impl Lexer {
     fn take_strterm(&mut self) -> StringLiteral {
-        match self.strterm.take() {
+        match self.strterm.take().map(|v| *v) {
             Some(StrTerm::StringLiteral(s)) => s,
             _ => unreachable!("strterm must be string"),
         }
     }
     fn restore_strterm(&mut self, literal: StringLiteral) {
-        self.strterm = Some(StrTerm::StringLiteral(literal));
+        self.strterm = Some(Box::new(StrTerm::StringLiteral(literal)));
     }
 
-    fn parse_string(&mut self) -> i32 {
+    pub(crate) fn parse_string(&mut self) -> i32 {
         let mut quote = self.take_strterm();
 
         let func = quote.func;
@@ -106,7 +73,7 @@ impl ParseString for Lexer {
         if c == term && quote.nest == 0 {
             if (func & STR_FUNC_QWORDS) != 0 {
                 quote.func |= STR_FUNC_TERM;
-                self.buffer.pushback(&c); /* dispatch the term at tSTRING_END */
+                self.buffer.pushback(c); /* dispatch the term at tSTRING_END */
                 self.restore_strterm(quote);
                 return Self::tSPACE;
             }
@@ -114,7 +81,7 @@ impl ParseString for Lexer {
             return self.string_term(term, func);
         }
         if space {
-            self.buffer.pushback(&c);
+            self.buffer.pushback(c);
             self.restore_strterm(quote);
             return Self::tSPACE;
         }
@@ -127,7 +94,7 @@ impl ParseString for Lexer {
             self.tokadd(b'#');
             c = self.nextc();
         }
-        self.buffer.pushback(&c);
+        self.buffer.pushback(c);
 
         let mut nest = quote.nest;
         let added = self.tokadd_string(func, term, paren, &mut nest);
@@ -182,7 +149,7 @@ impl ParseString for Lexer {
         loop {
             c = self.nextc();
 
-            let ch = match c.to_option() {
+            let ch = match c.as_option() {
                 Some(_) if !c.is_alpha() => break,
                 None => break,
                 Some(ch) => ch,
@@ -193,12 +160,12 @@ impl ParseString for Lexer {
                     result.push(ch as char);
                 }
                 _ => {
-                    self.tokadd(&c);
+                    self.tokadd(c);
                 }
             }
         }
 
-        self.buffer.pushback(&c);
+        self.buffer.pushback(c);
         if self.toklen() > 0 {
             self.tokfix();
             self.compile_error(
@@ -216,7 +183,7 @@ impl ParseString for Lexer {
         result
     }
 
-    fn peek_variable_name(&mut self) -> Option<i32> {
+    pub(crate) fn peek_variable_name(&mut self) -> Option<i32> {
         let mut c: MaybeByte;
         let mut ptr: usize = self.buffer.pcur;
 
@@ -226,7 +193,7 @@ impl ParseString for Lexer {
         c = self.char_at(ptr);
         ptr += 1;
 
-        match c.to_option() {
+        match c.as_option() {
             Some(b'$') => {
                 c = self.char_at(ptr);
                 if c == b'-' {
@@ -267,7 +234,7 @@ impl ParseString for Lexer {
         None
     }
 
-    fn tokadd_string(
+    pub(crate) fn tokadd_string(
         &mut self,
         func: usize,
         term: u8,
@@ -284,14 +251,14 @@ impl ParseString for Lexer {
             }
 
             if self.buffer.heredoc_indent > 0 {
-                self.update_heredoc_indent(&c);
+                self.update_heredoc_indent(c);
             }
 
             if c == paren {
                 *nest += 1;
             } else if c == term {
                 if *nest == 0 {
-                    self.buffer.pushback(&c);
+                    self.buffer.pushback(c);
                     break;
                 }
                 *nest -= 1;
@@ -301,13 +268,13 @@ impl ParseString for Lexer {
             {
                 let c2 = self.char_at(self.buffer.pcur);
                 if c2 == b'$' || c2 == b'@' || c2 == b'{' {
-                    self.buffer.pushback(&c);
+                    self.buffer.pushback(c);
                     break;
                 }
             } else if c == b'\\' {
                 self.literal_flush(self.buffer.pcur - 1);
                 c = self.nextc();
-                match c.to_option() {
+                match c.as_option() {
                     Some(b'\n') => {
                         if (func & STR_FUNC_QWORDS) != 0 {
                             // break;
@@ -325,7 +292,7 @@ impl ParseString for Lexer {
                     }
                     Some(b'\\') => {
                         if (func & STR_FUNC_ESCAPE) != 0 {
-                            self.tokadd(&c)
+                            self.tokadd(c)
                         }
                     }
                     Some(b'u') => {
@@ -346,20 +313,20 @@ impl ParseString for Lexer {
                     _ => {
                         if !c.is_ascii() && (func & STR_FUNC_EXPAND) == 0 {
                             self.tokadd(b'\\');
-                            self.tokadd(&c);
+                            self.tokadd(c);
                         }
                         if (func & STR_FUNC_REGEXP) != 0 {
-                            if c == term && !self.simple_re_meta(&c) {
-                                self.tokadd(&c);
+                            if c == term && !self.simple_re_meta(c) {
+                                self.tokadd(c);
                                 continue;
                             }
-                            self.buffer.pushback(&c);
+                            self.buffer.pushback(c);
                             if self.tokadd_escape().is_err() {
                                 return None;
                             }
                             continue;
                         } else if (func & STR_FUNC_EXPAND) != 0 {
-                            self.buffer.pushback(&c);
+                            self.buffer.pushback(c);
                             if (func & STR_FUNC_ESCAPE) != 0 {
                                 self.tokadd(b'\\')
                             }
@@ -371,25 +338,25 @@ impl ParseString for Lexer {
                             // ignore backslashed spaces in %w
                         } else if c != term && c != paren {
                             self.tokadd(b'\\');
-                            self.buffer.pushback(&c);
+                            self.buffer.pushback(c);
                             continue;
                         }
                     }
                 }
             } else if !self.is_ascii() {
-                self.tokadd(&c);
+                self.tokadd(c);
                 continue;
             } else if (func & STR_FUNC_QWORDS) != 0 && c.is_space() {
-                self.buffer.pushback(&c);
+                self.buffer.pushback(c);
                 break;
             }
-            self.tokadd(&c);
+            self.tokadd(c);
         }
 
         Some(c)
     }
 
-    fn flush_string_content(&mut self) {
+    pub(crate) fn flush_string_content(&mut self) {
         // noop
     }
 
@@ -406,7 +373,7 @@ impl ParseString for Lexer {
         let mut result = 0;
 
         for _ in 0..len {
-            match self.buffer.byte_at(s).to_option() {
+            match self.buffer.byte_at(s).as_option() {
                 None => break,
                 Some(c) => match usize::from_str_radix(&(c as char).to_string(), 16) {
                     Ok(hex) => {
@@ -428,8 +395,8 @@ impl ParseString for Lexer {
         let mut result: usize = 0;
 
         for _ in 0..len {
-            match self.buffer.byte_at(s).to_option() {
-                Some(c) if (c >= b'0' && c <= b'7') => {
+            match self.buffer.byte_at(s).as_option() {
+                Some(c) if (b'0'..=b'7').contains(&c) => {
                     result <<= 3;
                     result |= (c - b'0') as usize;
                 }
@@ -442,12 +409,12 @@ impl ParseString for Lexer {
         result
     }
 
-    fn tokcopy(&mut self, n: usize) {
+    pub(crate) fn tokcopy(&mut self, n: usize) {
         let substr = self
             .buffer
             .substr_at(self.buffer.pcur - n, self.buffer.pcur)
             .unwrap_or_else(|| panic!("no substr {}..{}", self.buffer.pcur - n, self.buffer.pcur));
-        self.tokenbuf.append(&substr);
+        self.tokenbuf.append(substr);
     }
 
     fn tokaddmbc(&mut self, codepoint: usize) {
@@ -506,7 +473,12 @@ impl ParseString for Lexer {
         true
     }
 
-    fn tokadd_utf8(&mut self, term: Option<u8>, _symbol_literal: usize, regexp_literal: usize) {
+    pub(crate) fn tokadd_utf8(
+        &mut self,
+        term: Option<u8>,
+        _symbol_literal: usize,
+        regexp_literal: usize,
+    ) {
         let open_brace = b'{';
         let close_brace = b'}';
         let mut err_multiple_codepoints = false;
@@ -541,7 +513,7 @@ impl ParseString for Lexer {
                     second = Some(self.buffer.pcur);
                 }
                 if regexp_literal != 0 {
-                    self.tokadd(&last)
+                    self.tokadd(last)
                 }
                 if !self.tokadd_codepoint(regexp_literal, true) {
                     break;
@@ -585,20 +557,20 @@ impl ParseString for Lexer {
         }
     }
 
-    fn simple_re_meta(&mut self, c: &MaybeByte) -> bool {
+    fn simple_re_meta(&mut self, c: MaybeByte) -> bool {
         matches!(
-            c.to_option(),
-            Some(b'$')
-                | Some(b'*')
-                | Some(b'+')
-                | Some(b'.')
-                | Some(b'?')
-                | Some(b'^')
-                | Some(b'|')
-                | Some(b')')
-                | Some(b']')
-                | Some(b'}')
-                | Some(b'>')
+            c,
+            MaybeByte::Some(b'$')
+                | MaybeByte::Some(b'*')
+                | MaybeByte::Some(b'+')
+                | MaybeByte::Some(b'.')
+                | MaybeByte::Some(b'?')
+                | MaybeByte::Some(b'^')
+                | MaybeByte::Some(b'|')
+                | MaybeByte::Some(b')')
+                | MaybeByte::Some(b']')
+                | MaybeByte::Some(b'}')
+                | MaybeByte::Some(b'>')
         )
     }
 
@@ -615,7 +587,7 @@ impl ParseString for Lexer {
 
         loop {
             c = self.nextc();
-            match c.to_option() {
+            match c.as_option() {
                 Some(b'\n') => return Ok(()),
 
                 Some(b'0') | Some(b'1') | Some(b'2') | Some(b'3') | Some(b'4') | Some(b'5')
@@ -642,7 +614,7 @@ impl ParseString for Lexer {
                     }
                     c = self.nextc();
                     if c != b'-' {
-                        self.buffer.pushback(&c);
+                        self.buffer.pushback(c);
                         return self.tokadd_escape_eof();
                     }
                     self.tokcopy(3);
@@ -655,7 +627,7 @@ impl ParseString for Lexer {
                     } else if c.is_eof() {
                         return self.tokadd_escape_eof();
                     }
-                    self.tokadd(&c);
+                    self.tokadd(c);
                     return Ok(());
                 }
 
@@ -665,7 +637,7 @@ impl ParseString for Lexer {
                     }
                     c = self.nextc();
                     if c != b'-' {
-                        self.buffer.pushback(&c);
+                        self.buffer.pushback(c);
                         return self.tokadd_escape_eof();
                     }
                     self.tokcopy(3);
@@ -677,7 +649,7 @@ impl ParseString for Lexer {
                     } else if c.is_eof() {
                         return self.tokadd_escape_eof();
                     }
-                    self.tokadd(&c);
+                    self.tokadd(c);
                     return Ok(());
                 }
 
@@ -695,7 +667,7 @@ impl ParseString for Lexer {
                     } else if c.is_eof() {
                         return self.tokadd_escape_eof();
                     }
-                    self.tokadd(&c);
+                    self.tokadd(c);
                     return Ok(());
                 }
 
@@ -733,12 +705,12 @@ impl ParseString for Lexer {
         MaybeByte::new(c as u8)
     }
 
-    fn read_escape(&mut self, flags: usize) -> MaybeByte {
+    pub(crate) fn read_escape(&mut self, flags: usize) -> MaybeByte {
         let mut c;
         let mut numlen: usize = 0;
 
         c = self.nextc();
-        match c.to_option() {
+        match c.as_option() {
             Some(b'\\') => c,
             Some(b'n') => MaybeByte::new(b'\n'),
             Some(b't') => MaybeByte::new(b'\t'),
@@ -750,7 +722,7 @@ impl ParseString for Lexer {
 
             Some(b'0') | Some(b'1') | Some(b'2') | Some(b'3') | Some(b'4') | Some(b'5')
             | Some(b'6') | Some(b'7') | Some(b'8') | Some(b'9') => {
-                self.buffer.pushback(&c);
+                self.buffer.pushback(c);
                 let c = self.scan_oct(self.buffer.pcur, 3, &mut numlen);
                 self.buffer.pcur += numlen;
                 MaybeByte::new(c as u8)
@@ -843,11 +815,11 @@ impl ParseString for Lexer {
         }
     }
 
-    fn is_ascii(&self) -> bool {
+    pub(crate) fn is_ascii(&self) -> bool {
         self.char_at(self.buffer.pcur - 1).is_ascii()
     }
 
-    fn warn_space_char(&mut self, c: u8, prefix: &'static str) {
+    pub(crate) fn warn_space_char(&mut self, c: u8, prefix: &'static str) {
         self.warn(
             DiagnosticMessage::new_invalid_character_syntax(format!("{}\\{}", prefix, c).into()),
             self.current_loc(),
