@@ -1,24 +1,17 @@
+crate::use_native_or_external!(List);
 use crate::lexer::*;
 use crate::Bytes;
 use crate::TokenBuf;
 
-pub(crate) trait Yylval {
-    fn set_yylval_id(&mut self, id: &str);
-    fn set_yylval_literal(&mut self, value: &TokenBuf);
-    fn set_yylval_num(&mut self, flags: String);
-    fn set_yylval_str(&mut self, value: &TokenBuf);
-    fn set_yylval_name(&mut self);
-}
-
-impl Yylval for Lexer {
-    fn set_yylval_id(&mut self, id: &str) {
+impl Lexer {
+    pub(crate) fn set_yylval_id(&mut self, id: &str) {
         if self.debug {
             println!("set_yylval_id({})", id);
         }
-        self.lval = Some(Bytes::new(id.as_bytes().to_vec()));
+        self.lval = Some(Bytes::new(List::from(id)));
     }
 
-    fn set_yylval_literal(&mut self, value: &TokenBuf) {
+    pub(crate) fn set_yylval_literal(&mut self, value: &TokenBuf) {
         if self.debug {
             println!(
                 "set_yylval_literal({:#?}) ptok = {}, pcur = {}",
@@ -28,21 +21,21 @@ impl Yylval for Lexer {
         self.lval = Some(value.bytes.clone());
     }
 
-    fn set_yylval_num(&mut self, flags: String) {
+    pub(crate) fn set_yylval_num(&mut self, flags: String) {
         if self.debug {
             println!("set_yylval_num {:#?}", flags);
         }
-        self.lval = Some(Bytes::new(flags.into_bytes()));
+        self.lval = Some(Bytes::new(List::from(flags)));
     }
 
-    fn set_yylval_str(&mut self, value: &TokenBuf) {
+    pub(crate) fn set_yylval_str(&mut self, value: &TokenBuf) {
         if self.debug {
             println!("set_yylval_str {:#?}", value);
         }
         self.lval = Some(value.bytes.clone());
     }
 
-    fn set_yylval_name(&mut self) {
+    pub(crate) fn set_yylval_name(&mut self) {
         if self.debug {
             println!("set_yyval_name({:#?})", self.tokenbuf);
         }
