@@ -1,14 +1,15 @@
 crate::use_native_or_external!(StringPtr);
+crate::use_native_or_external!(Maybe);
 
 use crate::debug_level;
-use crate::source::maybe_token_rewriter::MaybeTokenRewriter;
-use crate::source::MaybeDecoder;
+use crate::source::token_rewriter::TokenRewriter;
+use crate::source::Decoder;
 
 #[repr(C)]
 pub(crate) struct InternalParserOptions {
     pub(crate) buffer_name: StringPtr,
     pub(crate) debug: debug_level::Type,
-    pub(crate) decoder: MaybeDecoder,
-    pub(crate) token_rewriter: MaybeTokenRewriter,
+    pub(crate) decoder: Maybe<Decoder>,
+    pub(crate) token_rewriter: Maybe<TokenRewriter>,
     pub(crate) record_tokens: bool,
 }
