@@ -1,7 +1,7 @@
+crate::use_native_or_external!(Maybe);
+
 use super::ParserOptions;
 use crate::debug_level;
-use crate::source::maybe_token_rewriter::{MaybeTokenRewriter, MaybeTokenRewriterAPI};
-use crate::source::{MaybeDecoder, MaybeDecoderAPI};
 
 const DEFAULT_BUFFER_NAME: &str = "(eval)";
 
@@ -10,8 +10,8 @@ impl Default for ParserOptions {
         Self::new(
             DEFAULT_BUFFER_NAME.to_string().into(),
             debug_level::NONE,
-            MaybeDecoder::new_none(),
-            MaybeTokenRewriter::new_none(),
+            Maybe::none(),
+            Maybe::none(),
             true,
         )
     }
