@@ -2,12 +2,13 @@ use std::convert::TryFrom;
 
 crate::use_native_or_external!(StringPtr);
 crate::use_native_or_external!(List);
+crate::use_native_or_external!(Maybe);
 
 use crate::debug_level;
 use crate::maybe_byte::*;
 use crate::source::input::Input;
+use crate::source::Decoder;
 use crate::source::InputError;
-use crate::source::MaybeDecoder;
 
 #[derive(Debug, Default)]
 pub(crate) struct Buffer {
@@ -30,15 +31,14 @@ pub(crate) struct Buffer {
     pub(crate) heredoc_line_indent: i32,
 
     pub(crate) tokidx: usize,
-    pub(crate) toksize: usize,
+    // pub(crate) toksize: usize,
     pub(crate) tokline: usize,
 
     pub(crate) has_shebang: bool,
 
-    pub(crate) ruby_sourceline: usize,     /* current line no. */
-    pub(crate) ruby_sourcefile: Vec<char>, /* current source file */
-    pub(crate) ruby_sourcefile_string: Vec<char>,
-
+    pub(crate) ruby_sourceline: usize, /* current line no. */
+    // pub(crate) ruby_sourcefile: Vec<char>, /* current source file */
+    // pub(crate) ruby_sourcefile_string: Vec<char>,
     pub(crate) debug: bool,
 }
 
@@ -46,7 +46,7 @@ impl Buffer {
     const CTRL_Z_CHAR: u8 = 0x1a;
     const CTRL_D_CHAR: u8 = 0x04;
 
-    pub(crate) fn new(name: StringPtr, bytes: List<u8>, decoder: MaybeDecoder) -> Self {
+    pub(crate) fn new(name: StringPtr, bytes: List<u8>, decoder: Maybe<Decoder>) -> Self {
         let mut input = Input::new(name, decoder);
 
         input.update_bytes(bytes);
@@ -63,7 +63,7 @@ impl Buffer {
 
     fn prepare(&mut self) {
         let c = self.nextc();
-        match c.to_option() {
+        match c.as_option() {
             Some(b'#') => {
                 if self.peek(b'!') {
                     self.has_shebang = true;
@@ -84,7 +84,7 @@ impl Buffer {
             _ => {}
         }
 
-        self.pushback(&c)
+        self.pushback(c)
     }
 
     pub(crate) fn nextc(&mut self) -> MaybeByte {
@@ -311,7 +311,7 @@ impl Buffer {
         self.pbeg = self.input.line_at(self.lastline).start();
         self.pend = self.pbeg + self.input.line_at(self.lastline).len();
         self.pcur = self.pend;
-        self.pushback(&MaybeByte::new(1));
+        self.pushback(1);
         self.set_ptok(self.pcur);
     }
 
@@ -352,23 +352,22 @@ impl Pushback<u8> for Buffer {
     }
 }
 
-impl Pushback<&Option<u8>> for Buffer {
-    fn pushback(&mut self, c: &Option<u8>) {
-        match c {
-            Some(c) => self.pushback(*c),
-            None => {}
+impl Pushback<Option<u8>> for Buffer {
+    fn pushback(&mut self, c: Option<u8>) {
+        if let Some(c) = c {
+            self.pushback(c)
         }
     }
 }
 
-impl Pushback<&MaybeByte> for Buffer {
-    fn pushback(&mut self, c: &MaybeByte) {
-        self.pushback(&c.to_option())
+impl Pushback<MaybeByte> for Buffer {
+    fn pushback(&mut self, c: MaybeByte) {
+        self.pushback(c.as_option())
     }
 }
 
 impl Pushback<&mut MaybeByte> for Buffer {
     fn pushback(&mut self, c: &mut MaybeByte) {
-        self.pushback(&c.to_option())
+        self.pushback(c.as_option())
     }
 }
